# Parts of this are borrowed from
# http://www.codekoala.com/blog/2009/aes-encryption-python-using-pycrypto/

import base64
from Crypto.Cipher import AES, PKCS1_OAEP
from Crypto.Hash import SHA
from Crypto.PublicKey import RSA
from Crypto.Signature import PKCS1_v1_5
import os

def sha1(s):
    h = SHA.new(s)
    return h.hexdigest()

AES_BLOCK_SIZE = 32
AES_PADDING = '{' # A random character that is used for padding to the block size
AES_ROOT_KEY = base64.b64encode("0" * AES_BLOCK_SIZE)

def base64_encode(s):
    return base64.urlsafe_b64encode(str(s))

def base64_decode(s):
    return base64.urlsafe_b64decode(str(s))

# one-liner to sufficiently pad the text to be encrypted
def _aes_pad(s):
    return s + (AES_BLOCK_SIZE - len(s) % AES_BLOCK_SIZE) * AES_PADDING

def aes_gen_key():
    return base64_encode(os.urandom(AES_BLOCK_SIZE))

def aes_gen_key_from_password(password):
    return base64_encode(_aes_pad(password))

def aes_encrypt(plaintext, key):
    key = base64_decode(key)
    cypher = AES.new(key)
    padded = _aes_pad(plaintext)
    encrypted = cypher.encrypt(str(padded))
    encoded = base64_encode(encrypted)
    return encoded

def aes_decrypt(cyphertext_encoded, key):
    key = base64_decode(key)
    cypher = AES.new(key)
    try:
        cyphertext = base64_decode(cyphertext_encoded)
    except Exception as e:
        raise e
    decrypted = cypher.decrypt(str(cyphertext)).rstrip(AES_PADDING)
    return str(decrypted)

RSA_KEY_SIZE = 2048
RSA_DUMMY_KEY = '-----BEGIN PUBLIC KEY-----\nMIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEAokq/Wwap1H+nmxmNqIYP\npzWG/rYEM/lqig+Xr1VE2WEeW6re5kGASfq9bzOvj7ZzMs2Rc7F5ME009FL+iBuX\n83wGzLMfUtamjfHMCyhOTGWXr1xg72sk7QOWLi5KGqH7B9seae3PPzpFU3cEK8lt\nXPO5w2ZTmreL7yZ/hoaYDGia0irz2kE1/IVTU72fb84AEhNKPh427QblN3QhLmAZ\nCWEt2fAPm79+mE0YNkOV+jTL5g97rRFl8oW8gC5S3eiqCJpEWPvZz/5Yf0mEoGv0\nfzqgYJZZ5lKsQBEuZYbApnLlM+TU8HWilSWk9LyQC53WGurMj6X2TNW9MhMs9iER\nkwIDAQAB\n-----END PUBLIC KEY-----'

def rsa_gen_keypair():
    keypair = RSA.generate(RSA_KEY_SIZE)
    pubkey = keypair.publickey()
    return keypair.exportKey('PEM'), pubkey.exportKey('PEM')

def rsa_encrypt(plaintext, pubkey):
    pubkey = RSA.importKey(pubkey)
    cypher = PKCS1_OAEP.new(pubkey)
    encrypted = cypher.encrypt(str(plaintext))
    encoded = base64_encode(encrypted)
    return str(encoded)

def rsa_decrypt(cyphertext_encoded, keypair):
    keypair = RSA.importKey(keypair)
    cypher = PKCS1_OAEP.new(keypair)
    cyphertext = base64_decode(cyphertext_encoded)
    decrypted = cypher.decrypt(str(cyphertext))
    return str(decrypted)

def rsa_sign(message, keypair):
    keypair = RSA.importKey(keypair)
    h = SHA.new(message)
    signer = PKCS1_v1_5.new(keypair)
    signature = signer.sign(h)
    encoded = base64_encode(signature)
    return encoded

def rsa_verify(message, signature_encoded, pubkey):
    pubkey = RSA.importKey(pubkey)
    h = SHA.new(message)
    signature = base64_decode(signature_encoded)
    verifier = PKCS1_v1_5.new(pubkey)
    return verifier.verify(h, signature)
